/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package wordsreminder;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Random;

/**
 * Class used to handle extended terms. It reads all tenses and build a list,
 * just like they were separate terms.
 * 
 * @author Jacek Koleyski
 */
public class Category {
	// ~ Instance fields
	// ------------------------------------------------------------------------------------------------

	/** The list of all extended, handled as separate terms. */
	private List<Subcategory> categories = new ArrayList<Subcategory>();

	/** Number of line read, just for debugging. */
	private int lineNr;

	// ~ Methods
	// --------------------------------------------------------------------------------------------------------

	/**
	 * Method returning a random extended of a random tense.
	 * 
	 * @return a term being one of the extended terms of one of the tenses
	 */
	public ITerm getRandomTerm() {
		Random r = new Random();
		Subcategory t = categories.get(r.nextInt(categories.size()));
		return t.getRandomTerm();
	}

	/**
	 * Method returning a random extended of a random tense.
	 * 
	 * @return a term being one of the extended terms of one of the tenses
	 */
	public Subcategory getRandomTense() {
		Random r = new Random();

		return categories.get(r.nextInt(categories.size()));
	}

	/**
	 * This method reads and parses the file and builds the structure of
	 * extended terms.
	 * 
	 * @param filename
	 *            the name of the file
	 * @param encoding
	 *            encoding used in the file
	 */
	public void readFromFile(String filename, String encoding) {
		BufferedReader in = null;

		lineNr = 0;

		try {
			in = new BufferedReader(new InputStreamReader(new FileInputStream(
					filename), encoding));

			String line = in.readLine();
			lineNr++;

			String categoryName = null;
			if ((line != null) && !line.equals("")
					&& line.contains(Reminder.extendedSeparatorFirst)) {
				categoryName = line.split(Reminder.extendedSeparatorFirst)[1]
						.trim();
			} else if ((line != null) && !line.equals("")
					&& line.contains(Reminder.extendedSeparatorSecond)) {
				String subcategoryName = line
						.split(Reminder.extendedSeparatorSecond)[1].trim();
				readTense(subcategoryName, in);
				return;
			}

			line = in.readLine();
			lineNr++;

			while ((line != null) && line.trim().equals("")) {
				line = in.readLine();
				lineNr++;
			}

			if ((line != null) && !line.equals("")
					&& line.contains(Reminder.extendedSeparatorSecond)) {
				String subcategoryName = line
						.split(Reminder.extendedSeparatorSecond)[1].trim();
				readTense(categoryName, subcategoryName, in);
			}
		} catch (Exception e) {
		} finally {
			try {
				in.close();
			} catch (IOException e) {

			}
		}
	}

	/**
	 * Helper method reading one tense and adding all extended terms to the
	 * list.
	 * 
	 * @param tense
	 *            name of the tense, used to build prefix displayed later in
	 *            application
	 * @param in
	 *            the buffered reader
	 */
	private void readTense(String tenseName, BufferedReader in) {
		readTense(null, tenseName, in);
	}

	/**
	 * Helper method reading one tense and adding all extended terms to the
	 * list.
	 * 
	 * @param tense
	 *            name of the tense, used to build prefix displayed later in
	 *            application
	 * @param in
	 *            the buffered reader
	 */
	private void readTense(String categoryName, String subcategoryName,
			BufferedReader in) {
		Subcategory subcategory = new Subcategory(categoryName, subcategoryName);
		try {
			String line = in.readLine();
			lineNr++;

			String prefix;
			if (categoryName != null)
				prefix = categoryName + ", " + subcategoryName;
			else
				prefix = subcategoryName;

			while (line != null) {
				if (line.contains(Reminder.extendedSeparatorSecond)) {
					subcategoryName = line
							.split(Reminder.extendedSeparatorSecond)[1].trim();
					readTense(categoryName, subcategoryName, in);
				} else if (line.trim().equals("")) {
					// do nothing, just skip the empty line
				} else {
					ITerm term = Utils.createTerm(prefix, line, lineNr);
					subcategory.add(term);
				}

				line = in.readLine();
				lineNr++;
			}
		} catch (Exception e) {
		}
		categories.add(subcategory);
	}

	public Subcategory getSubcategory(String name) {
		for (Iterator<Subcategory> iterator = categories.iterator(); iterator
				.hasNext();) {
			Subcategory sub = iterator.next();
			if (sub.getName().toLowerCase().contains(name.toLowerCase()))
				return sub;
		}
		return null;
	}
}
