/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package wordsreminder;

import java.util.HashSet;

/**
 * Implementation of ITerm, using a table of translations.
 * 
 * @author Jacek Koleyski
 */
public class CompositeTerm implements ITerm, Comparable {
	/** The word. */
	private String word;

	/** The translations. */
	private String[] translations;

	/**
	 * Creates a new ArrayTerm object.
	 * 
	 * @param word
	 *            the word
	 * @param translation
	 *            the translations
	 */
	public CompositeTerm(String word, String[] translation) {
		super();
		this.word = word;
		this.translations = translation;
	}

	/**
	 * Checks if the answer matches one of the translations.
	 * 
	 * @param answer
	 *            the answer to be checked
	 * 
	 * @return true if the answer is correct, false if not
	 */
	public boolean checkTranslation(String answer) {
		for (int i = 0; i < translations.length; i++) {
			if (this.translations[i].equalsIgnoreCase(answer)) {
				return true;
			}
		}

		return false;
	}

	/**
	 * Compares the term to another term. Compares only words, not translations.
	 * 
	 * @param obj
	 *            another term object to be compared
	 * 
	 * @return true if the terms are identical, false if not
	 */
	public int compareTo(Object obj) {
		ITerm st = (ITerm) obj;

		return word.compareTo(st.getWord());
	}

	/**
	 * Checks if two terms are equal. Two terms are considered equal if their
	 * words are identical, not translations.
	 * 
	 * @param obj
	 *            another term object to be compared
	 * 
	 * @return true if the terms are identical, false if not
	 */
	@Override
	public boolean equals(Object obj) {
		return word.equals(obj);
	}

	/**
	 * Gets a table of translations.
	 * 
	 * @return table of translations
	 */
	public String[] getTranslation() {
		return translations;
	}

	/**
	 * Gets translations concatenated to one string.
	 * 
	 * @return a string containing all translations
	 */
	public String getTranslationAsOneString() {
		StringBuilder builder = new StringBuilder();

		for (int i = 0; i < translations.length; i++) {
			builder.append(translations[i]);

			if (i != (translations.length - 1)) {
				builder.append(Reminder.translationSeparator);
				builder.append(" ");
			}
		}

		return builder.toString();
	}

	/**
	 * Gets the word.
	 * 
	 * @return the word
	 */
	public String getWord() {
		return word;
	}

	/**
	 * Returns the hashcode of the word.
	 * 
	 * @return the hashcode of the word
	 */
	@Override
	public int hashCode() {
		return word.hashCode();
	}

	/**
	 * Joins two terms. The output term contains all translations. Duplicates
	 * are eliminated.
	 * 
	 * @param term
	 *            the term to be joined
	 * 
	 * @return the output (newly created) term
	 */
	public ITerm join(ITerm term) {
		HashSet<String> set = new HashSet<String>();

		for (int i = 0; i < term.getTranslation().length; i++) {
			set.add(term.getTranslation()[i]);
		}

		for (int i = 0; i < getTranslation().length; i++) {
			set.add(getTranslation()[i]);
		}

		String[] newTranslations = set.toArray(new String[set.size()]);

		return new CompositeTerm(word, newTranslations);
	}

	/**
	 * Creates a string containing the word with all translations.
	 * 
	 * @return string containing the word with all translations
	 */
	@Override
	public String toString() {
		StringBuilder builder = new StringBuilder(word);
		builder.append(" ");
		builder.append(Reminder.termSeparator);
		builder.append(" ");
		builder.append(getTranslationAsOneString());

		return builder.toString();
	}
}
