/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package wordsreminder;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.util.Collection;
import java.util.Iterator;
import java.util.Map;
import java.util.TreeMap;

/**
 * FileDictionary class.
 * 
 * @author Jacek Koleyski
 */
public class FileDictionary extends AbstractFileDictionary {
	// ~ Instance fields
	// ------------------------------------------------------------------------------------------------

	// ~ Instance fields
	// ------------------------------------------------------------------------------------------------
	/** The dictionary. */
	protected Map<String, ITerm> dictionary = new TreeMap<String, ITerm>();

	// ~ Constructors
	// ---------------------------------------------------------------------------------------------------

	/**
	 * Creates a new FileDictionary object.
	 * 
	 * @param reminder
	 *            parent, Reminder
	 * @param fileName
	 *            the file name
	 * @param encoding
	 *            the encoding used
	 */
	private FileDictionary(Reminder reminder, String filename, String encoding) {
		super(reminder, filename, encoding);
	}

	// ~ Methods
	// --------------------------------------------------------------------------------------------------------

	/**
	 * Reads in the dictionary from a file.
	 * 
	 * @param reminder
	 *            parent, Reminder
	 * @param fileName
	 *            the name of the file
	 * @param encoding
	 *            the encoding used
	 * 
	 * @return FileDictionary object
	 */
	public static FileDictionary readDictionaryFromFile(Reminder reminder,
			String fileName, String encoding) {
		FileDictionary fd = new FileDictionary(reminder, fileName, encoding);
		BufferedReader in = null;
		boolean lateLoading = false;
		int lineNr = 0;

		try {
			in = new BufferedReader(new UnicodeReader(new FileInputStream(
					fileName), encoding));

			String line = in.readLine().trim();
			lineNr++;

			while (line != null) {
				if (!line.equals("")) {
					fd.addToDictionary(Utils.createTerm(line, lineNr));

					if (lineNr == 30) {
						new LateLoader(fd, in, fileName, lineNr, reminder)
								.start();
						lateLoading = true;

						break;
					}
				}

				line = in.readLine();
			}
		} catch (Exception e) {
			try {
				if (!lateLoading) {
					if (in != null) {
						in.close();
					}
				}
			} catch (IOException ex) {
			}

			reminder.showMessage(reminder.dictionaryFileErorString1 + fileName
					+ reminder.dictionaryFileErorString2, false);

			return null;
		}

		return fd;
	}

	/**
	 * {@inheritDoc}
	 */
	public synchronized ITerm addToDictionary(ITerm term) {
		if (term == null) {
			return null;
		}

		ITerm t;

		if (dictionary.containsKey(term.getWord())) {
			t = dictionary.remove(term.getWord());
			term = term.join(t);
		}

		dictionary.put(term.getWord(), term);

		return term;
	}

	/**
	 * {@inheritDoc}
	 */
	public synchronized ITerm removeFromDictionary(ITerm term) {
		if (term == null) {
			return null;
		}

		if (dictionary.containsKey(term.getWord())) {
			dictionary.remove(term.getWord());
		}

		return term;
	}

	/**
	 * {@inheritDoc}
	 */
	public synchronized ITerm getRandomTerm() {
		if (dictionary.size() > 0) {
			int i = random.nextInt(dictionary.size());

			ITerm term = (ITerm) dictionary.values().toArray()[i];

			return term;
		}

		return null;
	}

	/**
	 * {@inheritDoc}
	 */
	public synchronized void save() {
		String eol = System.getProperty("line.separator");

		try {
			BufferedWriter outWriter = new BufferedWriter(
					new OutputStreamWriter(new FileOutputStream(filename),
							encoding));

			Collection values = dictionary.values();
			Iterator iter = values.iterator();
			String line;

			while (iter.hasNext()) {
				ITerm term = (ITerm) iter.next();
				outWriter.write(term.toString() + eol);
			}

			outWriter.close();
		} catch (IOException e) {
		}
	}

	/**
	 * Creates an empty FileDictionary object.
	 * 
	 * @param reminder
	 *            the Reminder object
	 * @param filename
	 *            the file name
	 * @param encoding
	 *            the encoding used
	 * 
	 * @return IDictionary implementation
	 */
	public static FileDictionary createEmptyDictionary(Reminder reminder,
			String filename, String encoding) {
		return new FileDictionary(reminder, filename, encoding);
	}

	/**
	 * {@inheritDoc}
	 */
	public boolean contains(ITerm term) {
		if (term == null) {
			return false;
		}

		if (dictionary.containsKey(term.getWord())) {
			return true;
		}

		return false;
	}

	/**
	 * {@inheritDoc}
	 */
	public boolean isEmpty() {
		return dictionary.size() == 0;
	}

	/**
	 * @todo DOCUMENT ME!
	 */
	public void removeAll() {
		dictionary.clear();
	}
}

/**
 * This class performs a late loading of the dictionary.
 */
class LateLoader extends Thread {
	// ~ Instance fields
	// ------------------------------------------------------------------------------------------------

	/** The FileDictionary object. */
	private FileDictionary fileDictionary;

	/** The file name. */
	private String fileName;

	/** Buffered reader. */
	private BufferedReader in;

	/** Line number. */
	private int lineNr;

	/** The Reminder object. */
	Reminder reminder;

	// ~ Constructors
	// ---------------------------------------------------------------------------------------------------

	/**
	 * Creates a new LateLoader object.
	 * 
	 * @param fileDictionary
	 *            the FileDictionary object
	 * @param in
	 *            the BufferedReader object
	 * @param fileName
	 *            the file name
	 * @param lineNr
	 *            the line number
	 * @param wordsreminder
	 *            the Reminder object
	 */
	public LateLoader(FileDictionary fileDictionary, BufferedReader in,
			String fileName, int lineNr, Reminder reminder) {
		super();
		this.fileDictionary = fileDictionary;
		this.in = in;
		this.fileName = fileName;
		this.lineNr = lineNr;
		this.reminder = reminder;
	}

	// ~ Methods
	// --------------------------------------------------------------------------------------------------------

	/**
	 * Performs the late loading.
	 */
	@Override
	public void run() {
		int termsRead = 0;

		try {
			String line = in.readLine();
			lineNr++;

			while (line != null) {
				if (!line.equals("")) {
					fileDictionary.addToDictionary(Utils.createTerm(line,
							lineNr));
					termsRead++;
				}

				if ((lineNr % 50) == 0) {
					Thread.yield();
				}

				line = in.readLine();
			}

			in.close();
		} catch (Exception e) {
			reminder.showMessage(reminder.dictionaryFileErorString1 + fileName
					+ reminder.dictionaryFileErorString2, false);
		} finally {
			try {
				if (in != null) {
					in.close();
				}
			} catch (IOException ex) {
			}
		}
	}
}
