/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package wordsreminder;

import java.io.IOException;

/**
 * Implementation of IDictionary used for large files, impossible to be read
 * into memory.
 * 
 * @author Jacek Koleyski
 */
public class VeryBigFileDictionary extends AbstractFileDictionary {
	// ~ Instance fields
	// ------------------------------------------------------------------------------------------------

	/** The dictionary file representation. */
	private ExtendedRandomAccessFile dictionaryFile;

	// ~ Constructors
	// ---------------------------------------------------------------------------------------------------

	/**
	 * Creates a new FileDictionary object.
	 * 
	 * @param reminder
	 *            parent, Reminder
	 * @param fileName
	 *            the file name
	 * @param wordsreminder
	 *            the Reminder object
	 */
	private VeryBigFileDictionary(Reminder reminder, String fileName,
			String encoding) {
		super(reminder, fileName, encoding);
	}

	// ~ Methods
	// --------------------------------------------------------------------------------------------------------

	/**
	 * Adds a term to dictionary.
	 * 
	 * @param term
	 *            the term to be added
	 * 
	 * @return ITerm which has been added
	 */
	public synchronized ITerm addToDictionary(ITerm term) {
		if (term == null) {
			return null;
		}

		try {
			dictionaryFile.seek(dictionaryFile.length());
			dictionaryFile.writeBytes(term.toString());

			return term;
		} catch (IOException e) {
			return null;
		}
	}

	/**
	 * {@inheritDoc}
	 */
	public synchronized ITerm removeFromDictionary(ITerm term) {
		return term;
	}

	/**
	 * Gets a random term from dictionary.
	 * 
	 * @return a random ITerm or null if there is no term in this dictionary
	 */
	public synchronized ITerm getRandomTerm() {
		try {
			if ((dictionaryFile != null) && (dictionaryFile.length() > 0)) {
				// read first line, to find EOL
				String line = findLine();

				while ((line == null) || line.equals("")) {
					line = findLine();
				}

				return Utils.createTerm(line, -1);
			}
		} catch (Exception e) {
			reminder.showMessage(reminder.dictionaryFileErorString1 + filename
					+ reminder.dictionaryFileErorString2, false);

			return null;
		}

		return null;
	}

	/**
	 * Reads the dictionary from file.
	 * 
	 * @param reminder
	 *            the Reminder object
	 * @param filename
	 *            the file name
	 * @param encoding
	 *            the encoding
	 * 
	 * @return the IDictionary implementation
	 */
	public static VeryBigFileDictionary readDictionaryFromFile(
			Reminder reminder, String filename, String encoding) {
		VeryBigFileDictionary fd = new VeryBigFileDictionary(reminder,
				filename, encoding);

		try {
			fd.dictionaryFile = new ExtendedRandomAccessFile(filename, "rwd");

			String line = fd.dictionaryFile.readLine();

			if ((line == null) || line.equals("")) {
				reminder.showMessage(reminder.dictionaryFileErorString1
						+ filename + reminder.dictionaryFileErorString2, false);

				return null;
			}
		} catch (Exception e) {
			try {
				if (fd.dictionaryFile != null) {
					fd.dictionaryFile.close();
				}
			} catch (IOException ex) {
			}

			reminder.showMessage(reminder.dictionaryFileErorString1 + filename
					+ reminder.dictionaryFileErorString2 + ", ", false);

			return null;
		}

		return fd;
	}

	/**
	 * This implementation of IDictionary does not support saving.
	 */
	public synchronized void save() {
		// empty
	}

	/**
	 * Finds a random line in the dictionary.
	 * 
	 * @return a random line
	 */
	private String findLine() {
		try {
			double d = Math.random() * dictionaryFile.length();

			long l = (long) d;
			dictionaryFile.seek((long) d);

			String line = dictionaryFile.readLine();

			if (l > 5) {
				line = dictionaryFile.readLine(encoding);
			} else {
				dictionaryFile.seek(0);
				line = dictionaryFile.readLine(encoding);
			}

			return line;
		} catch (IOException e) {
			return null;
		}
	}

	/**
	 * {@inheritDoc}
	 */
	public boolean contains(ITerm term) {
		return false;
	}

	/**
	 * {@inheritDoc}
	 */
	public boolean isEmpty() {
		return false;
	}

	public void removeAll() {
		// empty
	}
}
